from __future__ import absolute_import, division

import numpy as np

from ..logger import msg
from ..constants import ZGLOBAL
from ..utils import area_of_polygon, getMid


def add_k0s(k0, mesh, prop_from_node, alpha, maxl_from_area, silent=True):
    msg('Adding K0s to K0...', silent=silent)
    dof = 5
    for tria in mesh.elements.values():
        # n1 -> n2 -> n3 -> n1
        # sub-tria1: mid -> n1 -> n2
        # sub-tria2: mid -> n2 -> n3
        # sub-tria3: mid -> n3 -> n1

        n1, n2, n3 = tria.nodes
        mid = getMid(tria)

        if np.dot(np.cross((n1.xyz - mid), (n2.xyz - mid)), ZGLOBAL) < 0:
            n1, n2, n3 = n2, n1, n3

        #TODO calculate triangle area here, in case nodes coordinate are updated
        #     during solution
        Ac = tria.get_area()

        # sub-tria 1
        xmid, ymid, zmid = mid
        x2, y2, z2 = n1.xyz
        x3, y3, z3 = n2.xyz
        a1 = x2 - xmid
        b1 = y2 - ymid
        c1 = x3 - xmid
        d1 = y3 - ymid
        tmp = np.array([mid, n1.xyz, n2.xyz])
        Ac1 = area_of_polygon(tmp[:, 0], tmp[:, 1])

        # sub-tria 2
        xmid, ymid, zmid = mid
        x2, y2, z2 = n2.xyz
        x3, y3, z3 = n3.xyz
        a2 = x2 - xmid
        b2 = y2 - ymid
        c2 = x3 - xmid
        d2 = y3 - ymid
        tmp = np.array([mid, n2.xyz, n3.xyz])
        Ac2 = area_of_polygon(tmp[:, 0], tmp[:, 1])

        # sub-tria 3
        xmid, ymid, zmid = mid
        x2, y2, z2 = n3.xyz
        x3, y3, z3 = n1.xyz
        a3 = x2 - xmid
        b3 = y2 - ymid
        c3 = x3 - xmid
        d3 = y3 - ymid

        tmp = np.array([mid, n3.xyz, n1.xyz])
        Ac3 = area_of_polygon(tmp[:, 0], tmp[:, 1])

        if prop_from_node:
            pn1 = n1.prop
            pn2 = n2.prop
            pn3 = n3.prop
            k13 = 1/3*pn1.scf_k13 + 1/3*pn2.scf_k13 + 1/3*pn3.scf_k13
            k23 = 1/3*pn1.scf_k23 + 1/3*pn2.scf_k23 + 1/3*pn3.scf_k23
            E = 1/3*pn1.E + 1/3*pn2.E + 1/3*pn3.E
            h = 1/3*pn1.h + 1/3*pn2.h + 1/3*pn3.h
        else:
            k13 = tria.prop.scf_k13
            k23 = tria.prop.scf_k23
            E = tria.prop.E
            h = tria.prop.h
        E44 = k13 * E[0, 0]
        E45 = 0 # min(k13, k23) * E[0, 1]
        E55 = k23 * E[1, 1]

        if maxl_from_area:
            maxl = Ac**0.5
        else:
            maxl = max([np.sum((e.n1.xyz - e.n2.xyz)**2)**0.5 for e in tria.edges])

        factor = alpha*maxl**2/h**2
        E44 = 1 / (1 + factor) * E44
        # E45 = 1 / (1 + factor) * E45
        E55 = 1 / (1 + factor) * E55

        i1 = n1.index
        i2 = n2.index
        i3 = n3.index

        k0[i1*dof+2, i1*dof+2] += (E44*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E45*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2)/Ac + (E45*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E55*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2)/Ac
        k0[i1*dof+2, i1*dof+3] += (E44*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E45*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac + (E45*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E55*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(-a1*c1/4 + a3*c3/4)/Ac
        k0[i1*dof+2, i1*dof+4] += (E44*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E45*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(b1*d1/4 - b3*d3/4)/Ac + (E45*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E55*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac
        k0[i1*dof+2, i2*dof+2] += (E44*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E45*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3)/Ac + (E45*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E55*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3)/Ac
        k0[i1*dof+2, i2*dof+3] += (E44*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E45*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac + (E45*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E55*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(a1*c1/4 - a2*c2/4)/Ac
        k0[i1*dof+2, i2*dof+4] += (E44*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E45*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(-b1*d1/4 + b2*d2/4)/Ac + (E45*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E55*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac
        k0[i1*dof+2, i3*dof+2] += (E44*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E45*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1)/Ac + (E45*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E55*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1)/Ac
        k0[i1*dof+2, i3*dof+3] += (E44*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E45*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac + (E45*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E55*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(a2*c2/4 - a3*c3/4)/Ac
        k0[i1*dof+2, i3*dof+4] += (E44*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E45*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(-b2*d2/4 + b3*d3/4)/Ac + (E45*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2) + E55*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac
        k0[i1*dof+3, i1*dof+2] += (E44*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E45*(-a1*c1/4 + a3*c3/4))*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2)/Ac + (E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E55*(-a1*c1/4 + a3*c3/4))*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2)/Ac
        k0[i1*dof+3, i1*dof+3] += (E44*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E45*(-a1*c1/4 + a3*c3/4))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac + (E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E55*(-a1*c1/4 + a3*c3/4))*(-a1*c1/4 + a3*c3/4)/Ac
        k0[i1*dof+3, i1*dof+4] += (E44*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E45*(-a1*c1/4 + a3*c3/4))*(b1*d1/4 - b3*d3/4)/Ac + (E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E55*(-a1*c1/4 + a3*c3/4))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac
        k0[i1*dof+3, i2*dof+2] += (E44*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E45*(-a1*c1/4 + a3*c3/4))*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3)/Ac + (E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E55*(-a1*c1/4 + a3*c3/4))*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3)/Ac
        k0[i1*dof+3, i2*dof+3] += (E44*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E45*(-a1*c1/4 + a3*c3/4))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac + (E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E55*(-a1*c1/4 + a3*c3/4))*(a1*c1/4 - a2*c2/4)/Ac
        k0[i1*dof+3, i2*dof+4] += (E44*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E45*(-a1*c1/4 + a3*c3/4))*(-b1*d1/4 + b2*d2/4)/Ac + (E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E55*(-a1*c1/4 + a3*c3/4))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac
        k0[i1*dof+3, i3*dof+2] += (E44*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E45*(-a1*c1/4 + a3*c3/4))*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1)/Ac + (E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E55*(-a1*c1/4 + a3*c3/4))*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1)/Ac
        k0[i1*dof+3, i3*dof+3] += (E44*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E45*(-a1*c1/4 + a3*c3/4))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac + (E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E55*(-a1*c1/4 + a3*c3/4))*(a2*c2/4 - a3*c3/4)/Ac
        k0[i1*dof+3, i3*dof+4] += (E44*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E45*(-a1*c1/4 + a3*c3/4))*(-b2*d2/4 + b3*d3/4)/Ac + (E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))) + E55*(-a1*c1/4 + a3*c3/4))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac
        k0[i1*dof+4, i1*dof+2] += (E44*(b1*d1/4 - b3*d3/4) + E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2)/Ac + (E45*(b1*d1/4 - b3*d3/4) + E55*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2)/Ac
        k0[i1*dof+4, i1*dof+3] += (E44*(b1*d1/4 - b3*d3/4) + E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac + (E45*(b1*d1/4 - b3*d3/4) + E55*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(-a1*c1/4 + a3*c3/4)/Ac
        k0[i1*dof+4, i1*dof+4] += (E44*(b1*d1/4 - b3*d3/4) + E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(b1*d1/4 - b3*d3/4)/Ac + (E45*(b1*d1/4 - b3*d3/4) + E55*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac
        k0[i1*dof+4, i2*dof+2] += (E44*(b1*d1/4 - b3*d3/4) + E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3)/Ac + (E45*(b1*d1/4 - b3*d3/4) + E55*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3)/Ac
        k0[i1*dof+4, i2*dof+3] += (E44*(b1*d1/4 - b3*d3/4) + E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac + (E45*(b1*d1/4 - b3*d3/4) + E55*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(a1*c1/4 - a2*c2/4)/Ac
        k0[i1*dof+4, i2*dof+4] += (E44*(b1*d1/4 - b3*d3/4) + E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(-b1*d1/4 + b2*d2/4)/Ac + (E45*(b1*d1/4 - b3*d3/4) + E55*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac
        k0[i1*dof+4, i3*dof+2] += (E44*(b1*d1/4 - b3*d3/4) + E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1)/Ac + (E45*(b1*d1/4 - b3*d3/4) + E55*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1)/Ac
        k0[i1*dof+4, i3*dof+3] += (E44*(b1*d1/4 - b3*d3/4) + E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac + (E45*(b1*d1/4 - b3*d3/4) + E55*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(a2*c2/4 - a3*c3/4)/Ac
        k0[i1*dof+4, i3*dof+4] += (E44*(b1*d1/4 - b3*d3/4) + E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(-b2*d2/4 + b3*d3/4)/Ac + (E45*(b1*d1/4 - b3*d3/4) + E55*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac
        k0[i2*dof+2, i1*dof+2] += (E44*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E45*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2)/Ac + (E45*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E55*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2)/Ac
        k0[i2*dof+2, i1*dof+3] += (E44*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E45*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac + (E45*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E55*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(-a1*c1/4 + a3*c3/4)/Ac
        k0[i2*dof+2, i1*dof+4] += (E44*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E45*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(b1*d1/4 - b3*d3/4)/Ac + (E45*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E55*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac
        k0[i2*dof+2, i2*dof+2] += (E44*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E45*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3)/Ac + (E45*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E55*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3)/Ac
        k0[i2*dof+2, i2*dof+3] += (E44*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E45*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac + (E45*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E55*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(a1*c1/4 - a2*c2/4)/Ac
        k0[i2*dof+2, i2*dof+4] += (E44*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E45*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(-b1*d1/4 + b2*d2/4)/Ac + (E45*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E55*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac
        k0[i2*dof+2, i3*dof+2] += (E44*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E45*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1)/Ac + (E45*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E55*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1)/Ac
        k0[i2*dof+2, i3*dof+3] += (E44*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E45*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac + (E45*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E55*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(a2*c2/4 - a3*c3/4)/Ac
        k0[i2*dof+2, i3*dof+4] += (E44*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E45*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(-b2*d2/4 + b3*d3/4)/Ac + (E45*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3) + E55*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac
        k0[i2*dof+3, i1*dof+2] += (E44*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E45*(a1*c1/4 - a2*c2/4))*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2)/Ac + (E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E55*(a1*c1/4 - a2*c2/4))*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2)/Ac
        k0[i2*dof+3, i1*dof+3] += (E44*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E45*(a1*c1/4 - a2*c2/4))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac + (E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E55*(a1*c1/4 - a2*c2/4))*(-a1*c1/4 + a3*c3/4)/Ac
        k0[i2*dof+3, i1*dof+4] += (E44*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E45*(a1*c1/4 - a2*c2/4))*(b1*d1/4 - b3*d3/4)/Ac + (E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E55*(a1*c1/4 - a2*c2/4))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac
        k0[i2*dof+3, i2*dof+2] += (E44*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E45*(a1*c1/4 - a2*c2/4))*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3)/Ac + (E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E55*(a1*c1/4 - a2*c2/4))*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3)/Ac
        k0[i2*dof+3, i2*dof+3] += (E44*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E45*(a1*c1/4 - a2*c2/4))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac + (E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E55*(a1*c1/4 - a2*c2/4))*(a1*c1/4 - a2*c2/4)/Ac
        k0[i2*dof+3, i2*dof+4] += (E44*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E45*(a1*c1/4 - a2*c2/4))*(-b1*d1/4 + b2*d2/4)/Ac + (E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E55*(a1*c1/4 - a2*c2/4))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac
        k0[i2*dof+3, i3*dof+2] += (E44*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E45*(a1*c1/4 - a2*c2/4))*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1)/Ac + (E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E55*(a1*c1/4 - a2*c2/4))*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1)/Ac
        k0[i2*dof+3, i3*dof+3] += (E44*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E45*(a1*c1/4 - a2*c2/4))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac + (E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E55*(a1*c1/4 - a2*c2/4))*(a2*c2/4 - a3*c3/4)/Ac
        k0[i2*dof+3, i3*dof+4] += (E44*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E45*(a1*c1/4 - a2*c2/4))*(-b2*d2/4 + b3*d3/4)/Ac + (E45*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3) + E55*(a1*c1/4 - a2*c2/4))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac
        k0[i2*dof+4, i1*dof+2] += (E44*(-b1*d1/4 + b2*d2/4) + E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2)/Ac + (E45*(-b1*d1/4 + b2*d2/4) + E55*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2)/Ac
        k0[i2*dof+4, i1*dof+3] += (E44*(-b1*d1/4 + b2*d2/4) + E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac + (E45*(-b1*d1/4 + b2*d2/4) + E55*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(-a1*c1/4 + a3*c3/4)/Ac
        k0[i2*dof+4, i1*dof+4] += (E44*(-b1*d1/4 + b2*d2/4) + E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(b1*d1/4 - b3*d3/4)/Ac + (E45*(-b1*d1/4 + b2*d2/4) + E55*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac
        k0[i2*dof+4, i2*dof+2] += (E44*(-b1*d1/4 + b2*d2/4) + E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3)/Ac + (E45*(-b1*d1/4 + b2*d2/4) + E55*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3)/Ac
        k0[i2*dof+4, i2*dof+3] += (E44*(-b1*d1/4 + b2*d2/4) + E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac + (E45*(-b1*d1/4 + b2*d2/4) + E55*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(a1*c1/4 - a2*c2/4)/Ac
        k0[i2*dof+4, i2*dof+4] += (E44*(-b1*d1/4 + b2*d2/4) + E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(-b1*d1/4 + b2*d2/4)/Ac + (E45*(-b1*d1/4 + b2*d2/4) + E55*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac
        k0[i2*dof+4, i3*dof+2] += (E44*(-b1*d1/4 + b2*d2/4) + E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1)/Ac + (E45*(-b1*d1/4 + b2*d2/4) + E55*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1)/Ac
        k0[i2*dof+4, i3*dof+3] += (E44*(-b1*d1/4 + b2*d2/4) + E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac + (E45*(-b1*d1/4 + b2*d2/4) + E55*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(a2*c2/4 - a3*c3/4)/Ac
        k0[i2*dof+4, i3*dof+4] += (E44*(-b1*d1/4 + b2*d2/4) + E45*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(-b2*d2/4 + b3*d3/4)/Ac + (E45*(-b1*d1/4 + b2*d2/4) + E55*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac
        k0[i3*dof+2, i1*dof+2] += (E44*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E45*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2)/Ac + (E45*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E55*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2)/Ac
        k0[i3*dof+2, i1*dof+3] += (E44*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E45*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac + (E45*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E55*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(-a1*c1/4 + a3*c3/4)/Ac
        k0[i3*dof+2, i1*dof+4] += (E44*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E45*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(b1*d1/4 - b3*d3/4)/Ac + (E45*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E55*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac
        k0[i3*dof+2, i2*dof+2] += (E44*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E45*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3)/Ac + (E45*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E55*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3)/Ac
        k0[i3*dof+2, i2*dof+3] += (E44*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E45*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac + (E45*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E55*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(a1*c1/4 - a2*c2/4)/Ac
        k0[i3*dof+2, i2*dof+4] += (E44*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E45*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(-b1*d1/4 + b2*d2/4)/Ac + (E45*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E55*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac
        k0[i3*dof+2, i3*dof+2] += (E44*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E45*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1)/Ac + (E45*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E55*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1)/Ac
        k0[i3*dof+2, i3*dof+3] += (E44*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E45*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac + (E45*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E55*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(a2*c2/4 - a3*c3/4)/Ac
        k0[i3*dof+2, i3*dof+4] += (E44*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E45*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(-b2*d2/4 + b3*d3/4)/Ac + (E45*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1) + E55*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac
        k0[i3*dof+3, i1*dof+2] += (E44*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E45*(a2*c2/4 - a3*c3/4))*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2)/Ac + (E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E55*(a2*c2/4 - a3*c3/4))*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2)/Ac
        k0[i3*dof+3, i1*dof+3] += (E44*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E45*(a2*c2/4 - a3*c3/4))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac + (E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E55*(a2*c2/4 - a3*c3/4))*(-a1*c1/4 + a3*c3/4)/Ac
        k0[i3*dof+3, i1*dof+4] += (E44*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E45*(a2*c2/4 - a3*c3/4))*(b1*d1/4 - b3*d3/4)/Ac + (E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E55*(a2*c2/4 - a3*c3/4))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac
        k0[i3*dof+3, i2*dof+2] += (E44*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E45*(a2*c2/4 - a3*c3/4))*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3)/Ac + (E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E55*(a2*c2/4 - a3*c3/4))*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3)/Ac
        k0[i3*dof+3, i2*dof+3] += (E44*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E45*(a2*c2/4 - a3*c3/4))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac + (E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E55*(a2*c2/4 - a3*c3/4))*(a1*c1/4 - a2*c2/4)/Ac
        k0[i3*dof+3, i2*dof+4] += (E44*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E45*(a2*c2/4 - a3*c3/4))*(-b1*d1/4 + b2*d2/4)/Ac + (E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E55*(a2*c2/4 - a3*c3/4))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac
        k0[i3*dof+3, i3*dof+2] += (E44*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E45*(a2*c2/4 - a3*c3/4))*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1)/Ac + (E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E55*(a2*c2/4 - a3*c3/4))*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1)/Ac
        k0[i3*dof+3, i3*dof+3] += (E44*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E45*(a2*c2/4 - a3*c3/4))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac + (E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E55*(a2*c2/4 - a3*c3/4))*(a2*c2/4 - a3*c3/4)/Ac
        k0[i3*dof+3, i3*dof+4] += (E44*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E45*(a2*c2/4 - a3*c3/4))*(-b2*d2/4 + b3*d3/4)/Ac + (E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3))) + E55*(a2*c2/4 - a3*c3/4))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac
        k0[i3*dof+4, i1*dof+2] += (E44*(-b2*d2/4 + b3*d3/4) + E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(Ac1*(d1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac3*(-b3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b2 - 0.166666666666667*d2)/Ac + (E45*(-b2*d2/4 + b3*d3/4) + E55*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(Ac1*(-c1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac3*(a3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a2 + 0.166666666666667*c2)/Ac
        k0[i3*dof+4, i1*dof+3] += (E44*(-b2*d2/4 + b3*d3/4) + E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac + (E45*(-b2*d2/4 + b3*d3/4) + E55*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(-a1*c1/4 + a3*c3/4)/Ac
        k0[i3*dof+4, i1*dof+4] += (E44*(-b2*d2/4 + b3*d3/4) + E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(b1*d1/4 - b3*d3/4)/Ac + (E45*(-b2*d2/4 + b3*d3/4) + E55*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + 0.166666666666667*Ac2 + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac
        k0[i3*dof+4, i2*dof+2] += (E44*(-b2*d2/4 + b3*d3/4) + E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(Ac1*(-b1/(2*Ac1) + 0.166666666666667*(b1 - d1)/Ac1) + Ac2*(d2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + 0.166666666666667*b3 - 0.166666666666667*d3)/Ac + (E45*(-b2*d2/4 + b3*d3/4) + E55*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(Ac1*(a1/(2*Ac1) + 0.166666666666667*(-a1 + c1)/Ac1) + Ac2*(-c2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) - 0.166666666666667*a3 + 0.166666666666667*c3)/Ac
        k0[i3*dof+4, i2*dof+3] += (E44*(-b2*d2/4 + b3*d3/4) + E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(Ac1*(0.166666666666667 - b1*c1/(4*Ac1)) + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac + (E45*(-b2*d2/4 + b3*d3/4) + E55*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(a1*c1/4 - a2*c2/4)/Ac
        k0[i3*dof+4, i2*dof+4] += (E44*(-b2*d2/4 + b3*d3/4) + E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(-b1*d1/4 + b2*d2/4)/Ac + (E45*(-b2*d2/4 + b3*d3/4) + E55*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(Ac1*(0.166666666666667 + a1*d1/(4*Ac1)) + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + 0.166666666666667*Ac3)/Ac
        k0[i3*dof+4, i3*dof+2] += (E44*(-b2*d2/4 + b3*d3/4) + E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(Ac2*(-b2/(2*Ac2) + 0.166666666666667*(b2 - d2)/Ac2) + Ac3*(d3/(2*Ac3) + 0.166666666666667*(b3 - d3)/Ac3) + 0.166666666666667*b1 - 0.166666666666667*d1)/Ac + (E45*(-b2*d2/4 + b3*d3/4) + E55*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(Ac2*(a2/(2*Ac2) + 0.166666666666667*(-a2 + c2)/Ac2) + Ac3*(-c3/(2*Ac3) + 0.166666666666667*(-a3 + c3)/Ac3) - 0.166666666666667*a1 + 0.166666666666667*c1)/Ac
        k0[i3*dof+4, i3*dof+3] += (E44*(-b2*d2/4 + b3*d3/4) + E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 - b2*c2/(4*Ac2)) + Ac3*(0.166666666666667 + a3*d3/(4*Ac3)))/Ac + (E45*(-b2*d2/4 + b3*d3/4) + E55*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(a2*c2/4 - a3*c3/4)/Ac
        k0[i3*dof+4, i3*dof+4] += (E44*(-b2*d2/4 + b3*d3/4) + E45*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(-b2*d2/4 + b3*d3/4)/Ac + (E45*(-b2*d2/4 + b3*d3/4) + E55*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3))))*(0.166666666666667*Ac1 + Ac2*(0.166666666666667 + a2*d2/(4*Ac2)) + Ac3*(0.166666666666667 - b3*c3/(4*Ac3)))/Ac

    msg('finished!', silent=silent)
    return k0
